@@ -2,8 +2,8 @@
 @Author  : Likianta <likianta@foxmail.com>
 @Module  : lk_logger.py
 @Created : 2018-00-00
-@Updated : 2020-08-30
-@Version : 3.5.20
+@Updated : 2020-10-19
+@Version : 3.6.3
 @Desc    :
 
 Abbreviations:
@@ -62,10 +62,6 @@ from typing import *
     将 base_print() 的 prefix 设置为定宽
     支持 tag 打印时的高亮显示, 并设定高亮颜色
     增加动态抹除的进度条显示方法
-    
-维护事项:
-    - MsgRecorder.show_important_messages() 中会跳过 '[TEMPRINT]' 这个 tag 的
-      detail info, 该 tag 与 live template - lktp 相关, 如有改动, 请更新这里.
 """
 
 
@@ -123,7 +119,7 @@ class CallerFinder:
     
     def find_caller_by_hierarchy(self, hierarchy):
         """ Find the source caller related to the target hierarchy.
-        
+
         caller frame hierarchy:
             | caller                     | hierarchy |
             | -------------------------- | --------- |
@@ -212,35 +208,36 @@ class PathManager:
 
 class MsgRecorder:
     
-    def __init__(self):
-        self.log_messages = []
-        self.tag_messages = {'D': {}, 'I': {}, 'W': {}, 'E': {}, 'C': {}}
-        # e.g. {'D': {'D2324': [<str msg>, ...], 'D5332': [...], ...}, ...}
+    def __init__(self, tag_record_level='I'):
+        builtin_tags = ('D', 'I', 'W', 'E', 'C')
+        
+        try:
+            pos = builtin_tags.index(tag_record_level)
+            self._tag_ignores = builtin_tags[:pos]
+        except IndexError:
+            self._tag_ignores = ()
+        
+        self._log_messages = []
+        self._tag_messages = {t: {} for t in builtin_tags}
+        #   e.g. {'D': {'D2324': [<str msg>, ...], 'D5332': [...], ...}, ...}
     
     def record(self, msg, tag=''):
-        # print('[LKTEST]', 'lk_logger.py:220', msg, tag)
-        
-        self.log_messages.append(msg)
+        self._log_messages.append(msg)
         if tag:
             tag_initial = tag.rsplit('[', 1)[1][0]
-            """
-            e.g.
+            """ e.g.
                 tag = '[E2345]' -> ['', 'E2345]'] -> 'E2345]' -> 'E'
                 tag = '[Writer][D5211]' -> ['[Writer]', 'D5211]'] -> 'D'
             """
-            # print('[LKTEST]', 'lk_logger.py:220', tag, tag_initial)
-            node = self.tag_messages.setdefault(tag_initial, {})
-            node = node.setdefault(tag, [])
-            node.append(msg)
+            if tag_initial not in self._tag_ignores:
+                node = self._tag_messages.setdefault(tag_initial, {})
+                node = node.setdefault(tag, [])
+                node.append(msg)
     
     def show_important_messages(self, show_details=True, output='console'):
         """
-        NOTE:
-            1. 不显示 DEBUG 级别和 TEMPRINT 的详细信息 (即便 show_details=True).
-            2. 在输出时排下序, 让相同标签的出现在一起.
+        NOTE: 在输出时排下序, 让相同标签的出现在一起.
         """
-        ignore = re.compile(r'\[TEMPRINT]|\[D\d\d\d\d]')
-        
         msg_container = []
         
         """
@@ -249,19 +246,16 @@ class MsgRecorder:
         """
         # details info
         if show_details:
-            for msg_chunk in self.tag_messages.values():  # type: dict
-                """
-                msg_chunk: {tag: [<str msg>]}
-                -> tag: e.g. 'D2324'
-                   msg: e.g. 'app.py:28  >>  main()  >>  [D2324] hello'
+            for msg_chunk in self._tag_messages.values():  # type: dict
+                """ {tag: [<str msg>]}
+                    -> tag: e.g. 'D2324'
+                       msg: e.g. 'app.py:28  >>  main()  >>  [D2324] hello'
                 """
                 for tag, msg_list in msg_chunk.items():
-                    if ignore.findall(tag):
-                        continue
                     msg_container.extend(msg_list)
         
         # summary info
-        for msg_chunk in self.tag_messages.values():  # type: dict
+        for msg_chunk in self._tag_messages.values():  # type: dict
             for tag, msg_list in msg_chunk.items():
                 one_msg = msg_list[0]
                 if (token := '\t>>\t') in one_msg:
@@ -283,7 +277,7 @@ class MsgRecorder:
                 file.write('\n'.join(msg_container))
         
         # also take record to log_messages. (for the future to do dump_log())
-        self.log_messages += msg_container
+        self._log_messages += msg_container
     
     def dump_log(self, log_path, launch_path):
         with open(log_path, encoding='utf-8', mode='w') as f:
@@ -295,7 +289,7 @@ script filename is {ossplit(launch_path)[1]}.
 
             """.strip(' ')
             f.write(prefix)
-            f.write('\n'.join(self.log_messages))
+            f.write('\n'.join(self._log_messages))
 
 
 # ------------------------------------------------------------------------------
@@ -414,52 +408,61 @@ class LKLogger:
         看到, 为了让指向到达外部函数 fx(), over() 函数中特意将层级参数上调为
         'parent' 了)
     """
-    terminal = None  # None means terminal to console.
-    
-    log_enable = True  # total logger switcher.
-    lite_mode = False  # TODO: more works need to be done.
-    
-    direct_caller = None
+    _code_tracker: dict
+    _direct_caller = None
     _self = 'self'  # direct caller hierarchy. 绝大多数情况下, 此值是固定不变的
     #   (始终为 'self'); 少数情况下, 例如开发者想要特意将 direct caller 指向其他
     #   调用者, 则可以操作此变量. 目前操作此变量的函数只有 LKLogger 内部的方法:
     #       self.over(), self.dump_log() and self.print_important_msg().
     
-    _start_time = 0
-    _end_time = 0
+    log_enable = True  # total logger switcher.
+    lite_mode = False  # TODO: more works need to be done.
+    terminal = None  # None means terminal to console.
+    _log_style: dict
     
-    def __init__(self):
-        self.finder = CallerFinder()
-        self.recorder = MsgRecorder()
-        self.ast_analyser = AstAnalyser()
-        self.path_manager = PathManager()
+    __start_time = 0
+    __end_time = 0
+    
+    _finder: CallerFinder
+    _tag_recorder: MsgRecorder
+    
+    _ast_analyser: AstAnalyser
+    _path_mgr: PathManager
+    
+    def __init__(self, tag_record_level='I', enable_recorder=False):
+        self._finder = CallerFinder()
+        self._ast_analyser = AstAnalyser()
+        self._path_mgr = PathManager()
+        
+        if enable_recorder:
+            self._recorder = MsgRecorder(tag_record_level)
+        else:
+            self._recorder = None
         
         # fmt: {source_filepath: {source_lineno: [direct_var_name]}}
-        self.code_tracker = {}
+        self._code_tracker = {}
         
-        # FIXME: not ready to use.
-        self.log_style = {  # related to self.set_log_style
+        self._log_style = {  # related to self.set_log_style
             'show_func'          : True,
             'graphic_progressbar': False,
             'align_counter'      : True,
         }
         
         # start timing
-        self._start_time = time.time()
+        self.__start_time = time.time()
         print('start time = {}'.format(
-            _simple_timestamp('y-m-d h:n:s', self._start_time))
+            _simple_timestamp('y-m-d h:n:s', self.__start_time))
         )
-        self.recorder.record(self.path_manager.launch_path)
     
     def set_log_style(self, style: dict):
-        """ Overwrite self.log_style configurations.
-        
+        """ TODO: Overwrite self.log_style configurations.
+
         support properties:
             `show_func`: bool. default True.
                 True: e.g. printing 'myapp.py:12 >> main() >> hello'
                 False: e.g. printing 'myapp.py:12 >> hello'
             `graphic_progressbar`: bool. default False.
-                True: e.g. '■■■■-------------------------- 13%' (writing and 
+                True: e.g. '■■■■-------------------------- 13%' (writing and
                 swiping msg in one line.)
                 False: e.g.
                     [01/30] loop index count 1
@@ -469,10 +472,10 @@ class LKLogger:
             `align_counter`: bool. default True.
                 True: e.g. '[01/30] loop index count 1'
                 False: e.g. '[1/30] loop index count 1'
-            
+
         e.g. style = {'shor_func': False, 'align_counter': False}
         """
-        self.log_style.update(style)
+        self._log_style.update(style)
     
     # --------------------------------------------------------------------------
     
@@ -500,8 +503,8 @@ class LKLogger:
                 cnt, divide_line, tag, prtend
             )
         else:
-            filepath, lineno, function, _ = self.direct_caller
-            filepath = self.path_manager.get_relpath(filepath)
+            filepath, lineno, function, _ = self._direct_caller
+            filepath = self._path_mgr.get_relpath(filepath)
             if function != '<module>':
                 function += '()'
             
@@ -511,11 +514,10 @@ class LKLogger:
             )
         
         print(prefix, end='')
-        """
-        注意这里我们把 print 的 end 参数设为空字符串, 而由 prtend 控制是否在末尾
-        加空格. 这样做的目的是, 在 MsgRecorder 记录的时候, 也能记录到这个 rtend;
-        否则就会在 dump 的时候发现对齐不是很美观 (特别是涉及到 log*x, logd,
-        logt, logdt 的对齐的时候).
+        """ 注意这里我们把 print 的 end 参数设为空字符串, 而由 prtend 控制是否在
+            末尾加空格. 这样做的目的是, 在 MsgRecorder 记录的时候, 也能记录到这
+            个 rtend; 否则就会在 dump 的时候发现对齐不是很美观 (特别是涉及到
+            log*x, logd, logt, logdt 的对齐的时候).
         """
         
         # strip r-char
@@ -525,7 +527,8 @@ class LKLogger:
         
         # print it
         print(msg)
-        self.recorder.record(prefix + msg, tag=tag)
+        if self._recorder:
+            self._recorder.record(prefix + msg, tag=tag)
         return msg
     
     # --------------------------------------------------------------------------
@@ -537,7 +540,7 @@ class LKLogger:
     
     def loga(self, *data, h='self'):
         """ log in advanced mode.
-        
+
         NOTICE: 目前 (2019年7月7日) 无法解决以下问题:
             def foo(*x):
                 lk.loga(*x)
@@ -683,23 +686,23 @@ class LKLogger:
             return ''
     
     def over(self, total_count=0):
-        total_count = max(total_count, self._denominator, self.counter)
+        total_count = max((total_count, self._denominator, self.counter))
         
         self._self = 'parent'  # prompt
         h = 'parent'
         
         self.logd('计时结束', h=h)
-        self._end_time = time.time()
+        self.__end_time = time.time()
         
         self.log('开始运行: {}'.format(
-            _simple_timestamp('y-m-d h:n:s', self._start_time)
+            _simple_timestamp('y-m-d h:n:s', self.__start_time)
         ), h=h)
         self.log('结束运行: {}'.format(
-            _simple_timestamp('y-m-d h:n:s', self._end_time)
+            _simple_timestamp('y-m-d h:n:s', self.__end_time)
         ), h=h)
         
         # calculate duration
-        total_elapsed_time_sec = self._end_time - self._start_time
+        total_elapsed_time_sec = self.__end_time - self.__start_time
         if total_elapsed_time_sec < 0.01:
             duration = '{}ms'.format(round(total_elapsed_time_sec * 1000, 2))
         elif total_elapsed_time_sec < 60:
@@ -727,10 +730,11 @@ class LKLogger:
     # After things
     
     def print_important_msg(self, show_details=True, output='console'):
-        self._self = 'parent'  # prompt
-        self.logd('here collected important messages', h='parent')
-        self.recorder.show_important_messages(show_details, output)
-        self._self = 'self'  # reset
+        if self._recorder:
+            self._self = 'parent'  # prompt
+            self.logd('here collected important messages', h='parent')
+            self._recorder.show_important_messages(show_details, output)
+            self._self = 'self'  # reset
     
     def dump_log(self, output, timestamp=True):
         """
@@ -739,12 +743,15 @@ class LKLogger:
                 输出目标.
             timestamp: Add a time stamp.
         """
+        if not self._recorder:
+            return
+        
         if timestamp:
             from os.path import splitext
             a, b = splitext(output)
             output = f'{a}_{_simple_timestamp("ymd_hns")}.{b}'
         
-        self.recorder.dump_log(output, self.path_manager.launch_path)
+        self._recorder.dump_log(output, self._path_mgr.launch_path)
         self._self = 'parent'  # prompt
         self.log(f'log dumped at "{output}"', h='parent')
         self._self = 'self'  # reset
@@ -756,7 +763,7 @@ class LKLogger:
                            h: Union[str, int]) -> tuple:
         """ Analyse source code line, extract and orgnize params' names and
         values.
-        
+
         E.g.
             source code line: `lk.loga(name, gender, height, 'Engineer')`
             param data: ('Mike', 'Male', 180, 'Engineer')
@@ -783,18 +790,18 @@ class LKLogger:
             联 srcln. 有一个特殊情况, 如果 direct_caller 是 self.over(), 则需要
             将 direct_caller 向上提升一级.
         """
-        self.direct_caller = self.finder.find_caller_by_hierarchy(self._self)
-        srcln = self.direct_caller[-1]
+        self._direct_caller = self._finder.find_caller_by_hierarchy(self._self)
+        srcln = self._direct_caller[-1]
         
         # source caller
         if self._self == h:
-            filepath, lineno, _, _ = self.direct_caller
+            filepath, lineno, _, _ = self._direct_caller
         else:
-            filepath, lineno, _, _ = self.direct_caller = \
-                self.finder.find_caller_by_hierarchy(h)
+            filepath, lineno, _, _ = self._direct_caller = \
+                self._finder.find_caller_by_hierarchy(h)
         
         try:
-            return self.code_tracker[filepath][lineno]
+            return self._code_tracker[filepath][lineno]
         except KeyError:
             var_names = self._extract_vars_from_srcln(srcln)
             # print('[LKTEST]', 'lk_logger.py:810', srcln, var_names)
@@ -806,14 +813,14 @@ class LKLogger:
                     #   它剔除.
                     var_names.pop(0)
             
-            node = self.code_tracker.setdefault(filepath, {})
+            node = self._code_tracker.setdefault(filepath, {})
             node[lineno] = var_names
             
             return var_names
     
     def _extract_vars_from_srcln(self, raw_line):
         """ Extract variants' names from source code line.
-        
+
         IN: source_code_line (str): e.g. 'lk.loga(a, b, c, "hello")  # comment'
         OT: list or None. e.g. ['a', 'b', 'c', '']
             1. in this example, the 4th parameter has no variant name, so we
@@ -824,7 +831,7 @@ class LKLogger:
             3. if the source code line contains break lines,
                _get_var_names() **cannot handle it**. we have to return a bad
                result (-> None)
-                
+
         NOTE: This method will slow down the process, please use it as less as
             possible.
         """
@@ -833,12 +840,12 @@ class LKLogger:
                 raw_line = x[0]
             else:
                 return None
-        return self.ast_analyser.main(raw_line)
+        return self._ast_analyser.main(raw_line)
 
 
 def _simple_timestamp(style='y-m-d h:n:s', ctime=0.0) -> str:
     """ 生成时间戳.
-    
+
     转换关系:
         year  : %Y
         month : %m
@@ -846,7 +853,7 @@ def _simple_timestamp(style='y-m-d h:n:s', ctime=0.0) -> str:
         hour  : %H
         minute: %M
         second: %S
-    
+
     E.g. 'y-m-d h:n:s' -> '2018-12-27 15:13:45'
     """
     style = style \
